# -*- coding: utf-8 -*-


def parse_item_enum(parse_item, enum):
    """
    Args:
        parse_item:
        enum:
    """
    for i in enum:
        if parse_item == i.value:
            return i

    return enum.UNKNOWN


def parse_item_multi_value_enum(parse_item, enum):
    """
    Args:
        parse_item:
        enum:

    :return Currency
    """
    for i in enum:
        if parse_item in i.values:
            return i

    return enum.UNKNOWN



# -*- coding: utf-8 -*-
import cherrypy


class LightQiwiCallback(object):

    def __init__(self):
        cherrypy.quickstart(self, '/')

    @cherrypy.expose
    @cherrypy.tools.json_in()
    def index(self):
        data = cherrypy.request.json
        print(data)



# -*- coding: utf-8 -*-


class RequestError:

    def __init__(self, error_code):
        """
        Args:
            error_code:
        """
        if error_code == 400:
            raise RequestSyntaxError("Ошибка синтаксиса запроса (неправильный формат данных)")
        elif error_code == 401:
            raise InvalidTokenError("Неверный токен или истек срок действия токена")
        elif error_code == 403:
            raise RequestPermissionError(
                "Нет прав на данный запрос (недостаточно разрешений у токена)"
            )
        elif error_code == 404:
            raise APIError(
                "Список ошибок:https://developer.qiwi.com/ru/qiwi-wallet-personal/index.html?http#errors"
            )
        elif error_code == 422:
            raise ArgumentError("Неправильно указаны домен/подсеть/хост веб-хука(в параметре URL), "
                                "неправильно указаны тип хука или тип транзакции, "
                                "попытка создать хук при наличии уже созданного")
        elif error_code == 423:
            raise TooManyRequests("Слишком много запросов, сервис временно недоступен")
        elif error_code == 500:
            raise APIError("Внутренняя ошибка сервиса (превышена длина URL)")


class APIError(Exception):
    pass


class InvalidTokenError(Exception):
    pass


class ArgumentError(Exception):
    pass


class RequestSyntaxError(Exception):
    pass


class RequestPermissionError(Exception):
    pass


class TooManyRequests(Exception):
    pass



# -*- coding: utf-8 -*-
from Light_Qiwi.Enums import OperationType, Currency, PaymentStatus, Provider
from Light_Qiwi.Utils import parse_item_enum, parse_item_multi_value_enum


class Payment:

    def __init__(self, raw):
        """
        Объект полученный от
        https://developer.qiwi.com/ru/qiwi-wallet-personal/index.html?http#payments_list

        Args:
            raw (dict): Объект data от payment_history
        """
        self.raw = raw

        #: ID транзакции в процессинге QIWI Wallet
        self.transaction_id = raw['txnId']

        #: Клиентский ID транзакции
        self.trm_txn_id = raw['trmTxnId']

        #: Тип платежа (см. Enums.OperationType)
        self.type = parse_item_enum(raw['type'], OperationType)

        #: Номер кошелька
        self.phone = raw['personId']

        #: Код ошибки платежа
        self.error_code = raw['errorCode']

        #: Описание ошибки
        self.error = raw['error']

        #: Фактическая сумма платежа или пополнения
        self.amount = raw['total']['amount']

        #: Валюта платежа (см. Enums.CurrencyStr)
        self.currency = parse_item_multi_value_enum(
            raw['total']['currency'], Currency
        )

        #: Статус платежа (см. Enums.PaymentStatus)
        self.status = parse_item_enum(raw['status'], PaymentStatus)

        #: Текстовое описание статуса платежа
        self.status_text = raw['statusText']

        #: Дата/время платежа, во временной зоне запроса (см. параметр startDate)
        #: Формат даты ГГГГ-ММ-ДД'T'чч:мм:сс+03:00
        self.date = raw['date']

        #: Для платежей - номер счета получателя
        #: Для пополнений - номер отправителя, терминала или название агента пополнения кошелька
        self.account = raw['account']

        #: Комментарий к платежу
        self.comment = raw['comment']

        #: Комиссия платежа
        self.commission = raw['commission']['amount']

    def __eq__(self, other):
        if isinstance(other, Payment):
            return self.transaction_id == other.transaction_id

    def __str__(self):
        return str(self.raw)


class Balance:

    def __init__(self, raw):
        """
        Объект полученный от
        https://developer.qiwi.com/ru/qiwi-wallet-personal/index.html?http#balances_list

        Args:
            raw (dict): Объект account от balances_list
        """

        self.alias = raw['alias']
        self.balance = raw['balance']['amount']
        self.currency = parse_item_multi_value_enum(raw['currency'], Currency)

    def __str__(self):
        return str(self.balance)

    def __repr__(self):
        return "<Balance: {}, Alias: {}, Currency: {}>".format(
            self.balance, self.alias, self.currency
        )


class QiwiTransferAnswer:

    def __init__(self, raw):
        """
        Объект полученный от
        https://developer.qiwi.com/ru/qiwi-wallet-personal/index.html?http#balances_list

        Args:
            raw (dict): Объект account от balances_list
        """

        self.raw = raw
        self.id = raw['id']
        self.transaction_id = raw['transaction']['id']
        self.terms = parse_item_enum(raw['terms'], Provider)
        self.currency = parse_item_multi_value_enum(raw['transaction']['currency'], Currency)
        self.phone = raw['fields']['account']
        self.amount = raw['sum']['amount']
        self.source = raw['source']
        self.comment = raw['comment']
        self.status = raw['transaction']['state']['code']

    def __str__(self):
        return str(self.raw)



# -*- coding: utf-8 -*-
from enum import Enum

from aenum import MultiValueEnum


class OperationType(Enum):
    """
    Тип операций в отчете, для отбора.
    Используется тут:
        https://developer.qiwi.com/ru/qiwi-wallet-personal/index.html?http#payments_list
    """

    #: Все операции
    ALL = 'ALL'

    #: Только пополнения
    IN = 'IN'

    #: Только платежи
    OUT = 'OUT'

    #: Только платежи по картам QIWI (QVC, QVP)
    QIWI_CARD = 'QIWI_CARD'

    UNKNOWN = 'Unknown'


class Currency(MultiValueEnum):
    """
    Валюты
    """

    #: Рубли
    RUB = 'RUB', 643

    #: Доллары
    USD = 'USD', 840

    #: Евро
    EUR = 'EUR', 978

    UNKNOWN = 'Unknown', -1


class PaymentStatus(Enum):
    """
    Статус платежей
    """

    #: Платёж проводится
    WAITING = 'WAITING'

    #: Успешный платёж
    SUCCESS = 'SUCCESS'

    #: Ошибка платежа
    ERROR = 'ERROR'

    UNKNOWN = 'Unknown'


class Provider(Enum):
    """
    Провайдеры
    """

    #: Перевод на QIWI Wallet
    QIWI = 99

    #: Перевод на карту Visa (карты российских банков)
    RU_VISA = 1963

    #: Перевод на карту MasterCard (карты российских банков)
    RU_MASTER_CARD = 21013

    #: Для карт, выпущенных банками стран Азербайджан, Армения, Белоруссия,
    #: Грузия, Казахстан, Киргизия, Молдавия, Таджикистан, Туркменистан, Украина, Узбекистан
    ANOTHER_VISA = 1960
    ANOTHER_MASTER_CARD = 21012

    #: Перевод на карту национальной платежной системы МИР
    MIR = 31652

    UNKNOWN = 'Unknown'



# -*- coding: utf-8 -*-
from _thread import start_new_thread
from time import time, sleep

from requests import Session

from Light_Qiwi.Callback import *
from Light_Qiwi.Enums import *
from Light_Qiwi.Errors import *
from Light_Qiwi.Objects import *


class Qiwi:

    def __init__(self, token, phone):
        """
        :param token (str):
        :param phone (str):
        """

        self._s = Session()

        self._s.headers['Accept'] = 'application/json'
        self._s.headers['Content-Type'] = 'application/json'
        self._s.headers['Authorization'] = 'Bearer ' + token

        self._host = 'https://edge.qiwi.com/'

        self.phone = phone.replace('+', '')

        self.func = None
        self._bind_payments = []

    @property
    def balance(self):
        return self.get_balance()[0].balance

    @property
    def _transaction_id(self):
        """
        :return: UNIX time *
        1000
        """

        return str(int(time() * 1000))

    def pay(self, account: str, amount: float = 1.0, comment: str = "",
            currency: Currency = Currency.RUB, provider: Provider = Provider.QIWI):
        """
        Перевод денег на Qiwi кошелёк или Карты других банков.

        :type account: str
        :type amount: float
        :type comment: str
        :type currency: Currency
        :type provider: Provider

        :param comment: Комментарий к переводу
        :param currency: Валюта. Разрешены только рубли
        :param amount: Сумма перевода
        :param provider: Провайдер (см. Enums.Provider). Куда необходимо переводить деньги:
            на Qiwi, на Карту
        :param account: Номер получателя

        :raises: APIError

        :returns: Json Object
        :rtype: dict
        """
        url = self._host + 'sinap/api/v2/terms/{}/payments'.format(provider.value)

        json = {
            'id': self._transaction_id,
            'sum': {
                'amount': amount,
                'currency': str(currency.values[1])
            },
            'paymentMethod': {
                'type': 'Account',
                'accountId': '643'
            },
            'fields': {'account': account},
            'comment': comment
        }

        if provider != Provider.QIWI:
            del json['comment']

        response = self._req(url, json=json)
        data = response.json()

        if 'code' in data or 'errorCode' in data:
            data["status_code"] = response.status_code
            raise APIError(data)

        return data

    def _req(self, url, params: dict = None, json: dict = None):
        """
        Args:
            url:
            params (dict):
            json (dict):
        """
        if json:
            response = self._s.post(url, json=json)
        else:
            response = self._s.get(url, params=params)

        if response.status_code != 200 and not response.content:
            RequestError(response.status_code)

        return response

    def get_balance(self, retry=False):
        """
        Args:
            retry:
        """
        url = self._host + 'funding-sources/v2/persons/{}/accounts'.format(self.phone)
        params = {}

        if retry:
            params = {
                'timeout': 1000
            }

        response = self._req(url, params).json()

        balances = []

        for account in response['accounts']:
            if account['hasBalance']:
                if not account['balance']:
                    return self.get_balance(True)
                balances.append(Balance(account))

        return balances

    def _person_profile(self, auth_info=True, contract_info=True, user_info=True):
        """
        Args:
            auth_info:
            contract_info:
            user_info:
        """
        url = self._host + 'person-profile/v1/profile/current'

        params = {
            'authInfoEnabled': auth_info,
            'contractInfoEnabled': contract_info,
            'userInfoEnabled': user_info
        }

        return self._req(url, params)

    def identification(self):
        url = self._host + 'identification/v1/persons/{}/identification'.format(self.phone)

        return self._req(url).json()

    def get_payments(self, rows: int = 25, operation: OperationType = OperationType.ALL):
        """
        Args:
            rows (int):
            operation (OperationType):
        """
        url = self._host + 'payment-history/v2/persons/{}/payments'.format(self.phone)

        params = {
            'rows': rows,
            'operation': operation.value
        }
        
        data = self._req(url, params).json()
        
        if 'error' in data:
            raise APIError(data['error'])
        elif 'data' not in data:
            raise APIError(data)
        else:
            return [Payment(i) for i in data['data']]

    def get_transaction(self, transaction_id: int):
        """
        Args:
            transaction_id (int):
        """
        url = self._host + 'payment-history/v2/transactions/{}'.format(transaction_id)

        return Payment(self._req(url).json())

    @staticmethod
    def __callback():
        LightQiwiCallback()

    def check(self, interval: int = 3, amount: int = 25,
              operation: OperationType = OperationType.IN):
        """

        :param interval:
        :param amount:
        :param operation:
        :return:
        """
        payments = self.get_payments(operation=operation)

        while True:
            sleep(interval)

            new_payments = self.get_payments(amount, operation)
            update = False
            for payment in new_payments:
                if payment not in payments:
                    update = True
                    yield payment
            if update:
                payments = new_payments

    def bind_check(self, interval: int = 3, amount: int = 25,
                   operation: OperationType = OperationType.IN):
        """

        :param interval:
        :param amount:
        :param operation:
        :return decorator:
        """

        def decorator(func):
            if func.__code__.co_argcount != 1:
                raise ArgumentError('Функция должна иметь 1 аргумент!')

            def run():
                for payment in self.check(interval, amount, operation):
                    func(payment)

            self.func = run

        return decorator

    def start(self):
        return self.func()

    def start_threading(self):
        return start_new_thread(self.func, ())

    def on_payment_func(self, operation: OperationType = OperationType.IN):

        def decorator(func):
            if func.__code__.co_argcount != 1:
                raise ArgumentError('Функция должна иметь 1 аргумент!')

            @self.bind_check(5, 10, operation)
            def run(payment):
                if payment.comment in self._bind_payments:
                    self._bind_payments.remove(payment.comment)
                    func(payment)

            self.func = run

        return decorator

    def bind(self, comment):
        self._bind_payments.append(comment)



# -*- coding: utf-8 -*-
from setuptools import setup

"""
:authors: vffuunnyy
:license: Apache License, Version 2.0
:copyright: (c) 2019 vffuunnyy
"""

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='Light_Qiwi',
    version='1.2.0',
    packages=['Light_Qiwi'],
    url='https://github.com/vffuunnyy/Light_Qiwi',
    license='Apache License, Version 2.0',
    author='vffuunnyy',
    author_email='vffuunnyy@gmail.com',
    setup_requires=['pyversion'],
    auto_version=True,
    description=u'Python модуль для написания скриптов для Qiwi (qiwi.com) (API wrapper)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['requests', 'cherrypy', 'aenum'],
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ]
)
